/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_POSITION_H
#define DPC_IMPL_TYPE_POSITION_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"

namespace dpc {
struct position {
    ::Float x{ 0.0F };
    ::Float y{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(x);
        fun(y);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(x);
        fun(y);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("x", x);
        fun("y", y);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("x", x);
        fun("y", y);
    }

    friend bool operator==(const ::dpc::position& lhs, const ::dpc::position& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.x - rhs.x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.y - rhs.y)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_POSITION_H
